from infi.pyutils.enums import Enum, Value
from capacity import byte, TB, KiB, GiB
from .definitions import Definitions
import json, os

def _load_event_types_file():
    with open(_get_event_json_path(), 'r') as fd:
        data=json.load(fd)
    return data
def _get_event_json_path():
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), 'infinibox_event_defs.json')


latest = Definitions({
    "version": "2.0.0.0",
    "api" : {
        "default_page_size" : 50,
        "headers" : {
            "infinidat_approved_header_name" : "X-Infinidat-Approved",
            "infinidat_command_id_header_name" : "X-Infinidat-command-id",
            "infinidat_source_header_name" : "User-Agent",
        },
        "max_page_size" : 1000,
    },

    "auth": {
        "max_password_length": 50,
        "min_password_length": 0,
    },

    "internal": {
        "num_vus": 6,
        "section_size": 64 * KiB,
        "default_pool_reserved_capacity": 100 * GiB,
    },

    "events": {
        "max_description_template_length": 255,
        "events_types" :_load_event_types_file(),
    },

    "users": {
        "builtin_users": [
            {"id":-1, "username": "infinidat", "role": "Infinidat", "email": "dev.mgmt@infinidat.com", "password": "123456"},
            {"id":-2, "username": "admin", "role": "Admin", "email": "dev.mgmt@infinidat.com", "password": "123456"},
            {"id":-3, "username": "technician", "role": "Technician", "email": "dev.mgmt@infinidat.com", "password": "123456"},
        ]
    },
    "notifications": {
        "builtin_smtp_target_notifications": [
            {'from_address': 'no-reply@infinidat.com',
             'host': 'smtp-dev.lab.il.infinidat.com',
              'name': 'infinidat-smtp',
              'password': '',
              'port': 25,
              'protocol': 'SMTP',
              'tls': False,
              'username': '',
              'visibility': 'INFINIDAT'},
             {'from_address': 'no-reply@infinidat.com',
              'host': 'smtp',
              'name': 'customer-smtp',
              'password': '',
              'port': 25,
              'protocol': 'SMTP',
              'tls': False,
              'username': '',
              'visibility': 'CUSTOMER'}
            ]
    },
    "metadata": {
        "max_key_size": 255,
    },
    "enums": {
        "events": {
            "reporters": Enum("BLOCK", "FILE", "MGMT", "PLATFORM", "CUSTOM", "RMR"),
            "levels": Enum("INFO", "WARNING", "ERROR", "CRITICAL"),
            "visibilities": Enum("INFINIDAT", "CUSTOMER"),
            "codes": Enum(),
            "sort": Enum("ASC", "DESC"),
        },
        "email_rules": {
            "visibilities": Enum("INFINIDAT", "CUSTOMER"),
        },
        "hosts": {
            "port_types": Enum('ISCSI', 'FC'),
            "host_types": Enum("HP_UX", "DEFAULT"),
        },
        "enclosures": {
            "states": Enum('OK', 'WARNING', "ERROR", "MISSING", "UNKNOWN"),
            "drives": {
                "states": Enum("ACTIVE", "FAILED", "READY", "PHASING_OUT", "TESTING", "MISSING", "UNKNOWN"),
            },
        },
        "nodes": {
            "states": Enum("ACTIVE", "PHASING_IN", "PHASING_OUT", "FAILED", "DEGRADED", "READY", "ACTIVATING", "UNKNOWN"),
            "steady_states": Enum("ACTIVE", "READY", "DEGRADED", "FAILED"),
            "fc_ports": {
                "states":  Enum("OK", "WARNING", "ERROR", "MISSING", "UNKNOWN"),
                "link_states":  Enum("UP", "DOWN"),
                "roles": Enum('HARD_PORT', "SOFT_PORT"),
            },
            "drive_bays": {
                "states": Enum("POPULATED", "EMPTY"),
            },
            "drives": {
                "states": Enum(Value("ACTIVE", aliases=["OK"]), "WARNING", "ERROR", "MISSING", "UNKNOWN"),
                "fru_states": Enum("ACTIVE", "STANDBY", "NOTREADY", "MISSING"),
                "types": Enum("DISK", "SSD"),
            },
            "services": {
                "states": Enum("ACTIVE", "PROCESS_STARTED",
                               Value("FAILED", aliases=["PROCESS_FAILED", "PROCESS_FINISHED", "MISSING", "UNKNOWN"]),
                               Value("PHASED_OUT", aliases=["PROCESS_FINISHED", "MISSING", "UNKNOWN"]),
                               "MISSING", "INACTIVE"),
                "roles": Enum("MASTER", "SECONDARY", "MEMBER"),
                "depctl_states": Enum(Value('started', aliases=['mounted', 'loaded']),
                                      Value('finished', aliases=['stopped', 'unmounted', 'unloaded']),
                                      'failed',
                                      Value('core_dumping', aliases=['core dumping'])),
            },
        },
        "service_clusters": {
            "states": Enum('ACTIVE', 'INACTIVE'),
        },
        "system": {
            "states":Enum('ACTIVE', 'STARTING', 'STANDBY', 'NOT_READY', 'SHUTDOWN', 'EMERGENCY_SHUTDOWN', 'OFF'),
        },
        "pools": {
            "states": Enum("NORMAL", "LIMITED", "LOCKED"),

        },
        "volumes": {
            "types": Enum('MASTER', 'SNAP', 'CLONE'),
            "provisioning_types": Enum("THICK", "THIN"),
        },
        "filesystems": {
            "types": Enum('MASTER', 'SNAP', 'CLONE'),
            "provisioning_types": Enum("THICK", "THIN"),
        },
        "users": {
            "roles": Enum("Admin", "ReadOnly", "Technician", "Infinidat", "PoolAdmin"),
        },
        "notifications_targets": {
            "protocols": Enum("SNMP", "SMTP", "SYSLOG"),
            "visibilities": Enum("INFINIDAT", "CUSTOMER"),
            "snmp_auth_protocols": Enum("MD5", "SHA"),
            "snmp_private_protocols": Enum("DES", "AES"),
            "snmp_auth_types": Enum("NoAuthNoPriv", "AuthNoPriv", "AuthPriv"),
            "snmp_versions": Enum("SNMPv1", "SNMPv2c", "SNMPv3"),
            "syslog_transports": Enum("TCP", "UDP"),
            "syslog_facilities": Enum(*["local{0}".format(i) for i in range(8)]),
        },
        "network_spaces": {
            "services": Enum("NAS_SERVICE"),
        },
        "network_interfaces": {
            "states": Enum("OK", "degraded", "down"),
            # FIXME: when INFINIBOX-12968 will be resolved - Delete the above line & uncommned the line below
            #"states": Enum("CONNECTED", "DISCONNECTED", "PARTIAL_PG_FAILURE", "DISABLED"),
            "types": Enum("PORT", "PORT_GROUP"),
        },
    },
    "config" : {
        # TAKEN VERBATIM FROM http://.../api/rest/config
        "core": {
            "emergency.shutdown.mass_drive_loss.delay": 3600,
            "pool.physical_capacity.warning": 80,
            "port.dif_protection_type": True,
            "config.journal_shutdown_threshold": 90,
            "pool.physical_capacity.critical": 90,
            "port.xcopy_enabled": True,
            "emergency.shutdown.all_drive_loss.delay": 300,
            "config.allow_drive_model_mixing": False,
            "pool.physical_capacity.hysterezis_percent": 2,
        },
        "limits": {
            "cluster.max_clusters_in_system": 1000,
            "cluster.max_hosts_per_cluster": 32,
            "events.deletion_batch_size" : 100000,
            "events.maximum_saved_events" : 500000,
            "events.default_events_per_request": 50,
            "events.maximum_events_per_request": 1000,
            "events.max_email_queue_size": 30000,
            "hosts.max_hosts_in_system": 1000,
            "hosts.max_luns_per_map": 512,
            "hosts.max_ports_per_host": 16,
            "pools.min_snap_remaining_pool_space" : 0,
            "pools.max_pools_in_system": 1000,
            "pools.minimum_size": 1 * TB,
            "max_page_size" : 1000,
            "default_page_size" : 50,
            "GetHostByAddress.default_page_size" : 100,
            "metadata.max_in_system" : 1073741824,
            "metadata.max_per_entity" : 1024,
            "metadata.max_value_size" : 32672,
            "total.mappings": 250000,
            "volumes.max_tree_depth": 16,
            "volumes.max_volumes_in_system": 100000,
            "volumes.minimum_size": 1000000000 * byte,
            "volumes.resize_warning" : 2000000000000,
            "volumes.max_family_size": 250,
            'volumes.max_members_in_group_snapshot': 100,
            "users.max_users_in_system": 50,
            'system_virtual_capacity': 43809800060928,
            'notifications.max_rules_in_system': 10,
            'max_system_virtual_capacity': 43809800060928,
            'notifications.max_targets_in_system': 10,
            'soft_targets.max_soft_targets_in_system': 600,
            'soft_targets.max_soft_targets_in_switch': 128,
            'soft_targets.wwpn_range': 1024,
            'fc_switches.max_fc_switches_in_system': 16,
            'exports.max_exports_in_system': 10000,
            'exports.max_exports_on_file_system': 100,
            'file_systems.max_file_systems_in_system': 4000,
        },
        "mgmt": {
            'core.max.respawns.hour': 0,
            'environment.dns_servers': '192.168.8.20',
            'environment.locale': 'en_US',
            'environment.ntp_servers': 'ntp01.infinidat.com',
            'environment.timezone': 'UTC',
            'events.flooding_detector.enabled': False,
            'events.triggers.email.stop_timeout_seconds': 120,
            'mgmt.allow_single_node_mode': True,
            'mgmt.comm.link.disconnection': 10000,
            'mgmt.comm.link.keep.alive': 1000,
            'mgmt.core.activation.timeout': 60,
            'mgmt.core.emergencyshutdown.timeout': 600,
            'mgmt.core.phaseout.timeout': 1200,
            'mgmt.core.shutdown.timeout': 3600,
            'mgmt.core.standby.timeout': 3600,
            'mgmt.ignore_floating_ip_takeover': False,
            'mgmt.node.state.degraded.timeout': 10,
            'mgmt.node.state.init.timeout': 600,
            'mgmt.node.state.phasing_in.timeout': 7200,
            'mgmt.node.state.phasing_out.timeout': 600,
            'mgmt.reachable.ping': 2000,
            'mgmt.reachable.ssh': 1000,
            'mgmt.ups.activation.level': 50,
            'mgmt.ups.activation.safe.level': 95,
            'mgmt.ups.shutdown.level': 45,
            'mgmt.ups.warning.level': 50,
            'mgmt.wait_start_millis_1': 0,
            'mgmt.wait_start_millis_2': 0,
            'mgmt.wait_start_millis_3': 0,
            'mgmt.event.hwmonitor.grace.period': 300,
            'mgmt.event.log_anonymous_auth_attempts': False,
            'mgmt.emergency.shutdown.poweroff': False,
            'reset.password.token.encryption.password': u'1nf1n1pa$$',
            'reset.password.expiration': 3600,
            'generated.password.length': 6,
            'network.floating_ip': '172.16.64.145',
            'network.force_ssl': False,
            'network.fqdn': 'ibox512.lab.il.infinidat.com',
            'state.pause_startup_in': 'ACTIVE',
            'state.safe_activate_only': False,
            'mgmt.ldap.credentials_expiry': 600,
            'mgmt.emergency.shutdown.bbu.delay': 120,
            'mgmt.ldap.credentials_expiry': 600,
            'mgmt.bbu.poweroff.delay': 120,
            'mgmt.reachable.ping.attempts': 3,
            'mgmt.callhome.smtp.socket.connect.timeout': 30,
            'mgmt.emergency.shutdown.bbu.delay': 120,
            'pool.ssd_enabled_default': True,
            'mgmt.callhome.smtp.socket.timeout': 180,
            'mgmt.emergency.shutdown.with.core': False,
            'mgmt.ldap.auto_discovery_expiry': 600,
            'system.model_short_name': '',
            'system.model_long_name': '',
        },
        'ip_config': {
            'ip': '172.16.64.145',
            'netmask': '255.255.224.0',
            'gateway': '172.16.95.254'
        },
        "ssh" : {
            "ServerAliveCountMax": 2,
            "ServerAliveInterval": 1,
            "ConnectTimeout": 1,
            "ConnectionAttempts": 2,
        },
        "overriders" : {
        },
        "node_interfaces":[{"ip": "172.16.70.40", "netmask": "255.255.224.0", "node_id": 1},
                           {"ip": "172.16.65.150", "netmask": "255.255.224.0", "node_id": 2},
                           {"ip": "172.16.70.42", "netmask": "255.255.224.0", "node_id": 3}]
    }
})
