import infi.conf
from capacity import *
from infinibox_sysdefs.defs import latest as latest_sysdefs
from .__version__ import __version__

_LIMITS_CONFIG = latest_sysdefs["config"]["limits"]

config = infi.conf.Config(dict(
    caching=dict(
        strategy="none",
        timeout=120,
        ),
    defaults=dict(
        system_api_identifier="infinipy/" + __version__,
        system_api_timeout_seconds=30,
        system_api_port=80,
        system_api_ssl_port=443,
        system_api_log_level="DEBUG",
        system_api_auto_retry_sleep=5,
        volume=dict(
            size=1 * GB,
            ),
        pool=dict(
            size=1 * TB,
            ),
        pyformance_pattern='(?:^/api/rest/)?([^/\?\d]+)(?:/|\?.*|$)'
        ),
    constraints=dict(
        pool=dict(
            min_capacity=_LIMITS_CONFIG["pools.minimum_size"],
            min_increment=0 * TB,
            max_pools_in_system=_LIMITS_CONFIG["pools.max_pools_in_system"],
            ),
        volume=dict(
           min_capacity=_LIMITS_CONFIG["volumes.minimum_size"],
           min_increment=0 * GB,
           max_tree_depth=_LIMITS_CONFIG["volumes.max_tree_depth"],
           ),
        cluster=dict(
           max_hosts_per_cluster=_LIMITS_CONFIG["cluster.max_hosts_per_cluster"],
           max_clusters_in_system=_LIMITS_CONFIG["cluster.max_clusters_in_system"],
           ),
        host=dict(
           max_luns_per_map=_LIMITS_CONFIG["hosts.max_luns_per_map"],
           max_ports_per_host=_LIMITS_CONFIG["hosts.max_ports_per_host"],
           max_hosts=_LIMITS_CONFIG["hosts.max_hosts_in_system"],
           ),
        objects=dict(
           max_name_size=65,
           min_name_size=1,
           ),
        metadata = dict(
            max_value_size=_LIMITS_CONFIG["metadata.max_value_size"],
            max_per_entity=_LIMITS_CONFIG["metadata.max_per_entity"],
            max_in_system=_LIMITS_CONFIG["metadata.max_in_system"],
            max_key_size=255
            )
        ),
    timeouts=dict(
        system_shutdown=60 * 60,
        ),
    ))
