import logging
import socket

from urlobject import URLObject as URL

from ..config import config
from .exceptions import SystemException

_logger = logging.getLogger(__name__)

class AddressInfo(object):
    pass

class SimulatorAddressInfo(object):

    def __init__(self, address, port, use_ssl):
        super(SimulatorAddressInfo, self).__init__()
        self.simulator = self._resolve_simulator(address)
        assert self.simulator is not None
        self.hostname = self.simulator.get_hostname()
        self.ip_address = self.simulator.get_floating_addresses()[0]
        url = self.simulator.api.url
        if use_ssl:
            url = url.with_port(443)
        self.api_url = _maybe_ssl(url, use_ssl)

    def _resolve_simulator(self, address):
        try:
            import infinisim.core.simulator
            import infinisim.core.context
        except ImportError:
            _logger.debug("Infinisim not available")
            raise NotSimulator()

        if isinstance(address, infinisim.core.simulator.Simulator):
            return address

        _logger.debug("Address %r is not a simulator instance", address)
        simulator = infinisim.core.context.lookup_simulator_by_address(address)
        if simulator is None:
            raise NotSimulator()

        return simulator


class RealSystemAddressInfo(object):

    def __init__(self, address, port, use_ssl):
        super(RealSystemAddressInfo, self).__init__()
        self.simulator = None
        self.hostname = address
        self.ip_address = None

        try:
            self.ip_address = socket.gethostbyname(address)
        except socket.error, e:
            raise SystemException(
                "Cannot resolve system name {0!r} ({1})".format(address, e))

        if port is None:
            if use_ssl:
                port = config.root.defaults.system_api_ssl_port
            else:
                port = config.root.defaults.system_api_port

        self.api_url = _maybe_ssl(
            URL("http://{0}:{1}".format(address, port or config.root.defaults.system_api_port)),
            use_ssl)

class NotSimulator(Exception):
    pass

def _maybe_ssl(url, use_ssl):
    if use_ssl:
        return url.with_scheme("https")
    return url.with_scheme("http")
