from contextlib import contextmanager

from sentinels import NOTHING

old_username_passwords = []

class RequestSender(object):
    def request(self, method, path="", data=NOTHING, headers=(), raw_json_response=False):
        return self._request(method, path, data, headers, raw_json_response)
    def _request(self, method, path, data, headers, raw_json_response):
        raise NotImplementedError()  # pragma: no cover
    @contextmanager
    def get_modified_timeout_context(self, timeout_seconds):
        old = self.get_request_timeout()
        self.set_request_timeout(timeout_seconds)
        try:
            yield
        finally:
            self.set_request_timeout(old)
    def get_request_timeout(self):
        raise NotImplementedError()  # pragma: no cover
    def set_request_timeout(self, timeout_seconds):
        raise NotImplementedError()  # pragma: no cover
    def get(self, *args, **kw):
        return self.request('GET', *args, **kw).output
    def post(self, *args, **kw):
        return self.request('POST', *args, **kw).output
    def put(self, *args, **kw):
        return self.request('PUT', *args, **kw).output
    def delete(self, *args, **kw):
        return self.request('DELETE', *args, **kw).output
    def get_username_password(self):
        raise NotImplementedError()  # pragma: no cover
    def set_username_password(self, username, password):
        raise NotImplementedError()  # pragma: no cover
    def get_anonymous_context(self, **kwargs):
        return self.get_login_context(None, None, **kwargs)
    @contextmanager
    def get_login_context(self, username, password=None):
        """
        A context under which all commands are sent with a custom login and (optionally) a custom password.
        If password is None, it will be deduced from the Infinisim's known users
        """
        old_username_password = self.get_username_password()
        self.set_username_password(username, password)
        old_username_passwords.append(old_username_password)
        try:
            yield
        finally:
            useruser, oldpass = old_username_passwords.pop()
            self.set_username_password(useruser, oldpass)

    def get_command_id_context(self):
        raise NotImplementedError()  # pragma: no cover
    def get_unapproved_context(self):
        raise NotImplementedError()  # pragma: no cover
