from contextlib2 import ExitStack, contextmanager

class SystemCallbacks(object):
    def __init__(self, system):
        super(SystemCallbacks, self).__init__()
        for callback_name in [
            ]:
            setattr(self, callback_name, Callback(system))
        for contextback_name in [
            'snapshot_creation_context',
            'snapshot_restore_context',
            'volume_creation_context',
            'volume_move_context',
            'volume_resize_context',
            'volume_deletion_context',
            'pool_creation_context',
            'pool_deletion_context',
            'pool_resize_context',
            'api_command_context',
            ]:
            setattr(self, contextback_name, Contextback(system))

class Callback(object):
    def __init__(self, system):
        super(Callback, self).__init__()
        self._system = system
        self._callbacks = []
    def add_callback(self, callback):
        self._callbacks.append(callback)
    def __call__(self, *args, **kwargs):
        for callback in self._callbacks:
            callback(system = self._system, *args, **kwargs)

class Contextback(object):
    def __init__(self, system):
        super(Contextback, self).__init__()
        self._system = system
        self._contextbacks = []
    def add_contextback(self, context):
        self._contextbacks.append(context)
    @contextmanager
    def get_context(self, *args, **kwargs):
        with ExitStack() as stack:
            [stack.enter_context(ctx(system=self._system, *args, **kwargs)) for ctx in self._contextbacks]
            yield stack
    def remove_contextback(self, context):
        self._contextbacks.remove(context)

