from numbers import Number
import uuid
import capacity
from ..utils.default_arguments import has_defaults, DefaultValue, FromConfiguration

class FunctionBasedDefaultValue(DefaultValue):
    def __init__(self, func):
        super(FunctionBasedDefaultValue, self).__init__()
        self._func = func
    def get_argument_value(self, arg_name, arguments):
        return self._func(arg_name, arguments)

@FunctionBasedDefaultValue
def GENERATE_NAME(arg_name, arguments):
    object_typename = _deduce_object_typename_from_arguments(arguments)
    return '{0}_{1}'.format(object_typename.lower(), _random_str())

def GENERATE_NAME_WITH_PREFIX(prefix):
    @FunctionBasedDefaultValue
    def generator(arg_name, arguments):
        return "{0}_{1}".format(prefix, _random_str())
    return generator

def _random_str():
    return str(uuid.uuid1()).replace("-","")

def _deduce_object_typename_from_arguments(arguments):
    obj = arguments.get('self', None)
    if obj is not None:
        return obj.__class__.__name__
    objtype = arguments.get('cls', None)
    if objtype is None:
        return 'unknown'
    return objtype.__name__

class CapacityFromConfiguration(FromConfiguration):
    def get_argument_value(self, *args, **kwargs):
        returned = super(CapacityFromConfiguration, self).get_argument_value(*args, **kwargs)
        if isinstance(returned, Number):
            returned = returned * capacity.byte
        elif isinstance(returned, basestring):
            returned = capacity.from_string(returned)
        return returned

class GetDefaultFromMethod(DefaultValue):
    def __init__(self, method_name):
        super(GetDefaultFromMethod, self).__init__()
        self.method_name = method_name
    def get_argument_value(self, arg_name, arguments):
        self_argument = arguments['self']
        method = getattr(self_argument, self.method_name)
        return method(arg_name, arguments)
