from .system_object import CollectionSystemObject
import sys

class EmailRule(CollectionSystemObject):
    @classmethod
    def create(cls, system, name, visibility, recipients, filters=[]):
        return cls._create_from_post(system, dict(
                    name=name,
                    visibility=str(visibility),
                    recipients=recipients,
                    filters=[f.to_dict() for f in filters]))
    @classmethod
    def get_collection_path(cls):
        return "events/mail"
    # adding methods
    def add_filter(self, email_filter):
        system_filter = self._api.post('filters', email_filter.to_dict())
        return EmailFilter(**system_filter)
    def get_filters(self):
        return [EmailFilter(**email_filter) for email_filter in self._get_underlying_object()["filters"]]
    def set_filters(self, email_filters):
        filters_dict = [email_filter.to_dict() for email_filter in email_filters]
        return [EmailFilter(**email_filter) for email_filter in self._api.put('filters', filters_dict)]
    def delete_filter(self, email_filter):
        deleted_filter = self._api.delete('filters/{0}'.format(email_filter.get_id()))
        return EmailFilter(**deleted_filter)
    def add_recipient(self, recipient):
        return self._api.post('recipients', recipient)
    def get_recipients(self):
        return self._get_underlying_object()["recipients"]
    def set_recipients(self, recipients):
        return self._api.put('recipients', recipients)
    def delete_recipient(self, recipient):
        return self._api.delete('recipients/{0}'.format(recipient))
    def set_visibility(self, new_visibility):
        self._api.put('visibility', str(new_visibility))
    def get_visibility(self):
        return self._api.get('visibility')

class EmailFilter(object):
    def __init__(self, is_data, field_name, accepted_values, id=None):
        self._id = id
        self.is_data = is_data
        self.field_name = field_name
        self.accepted_values = accepted_values
    def get_id(self):
        return self._id
    def to_dict(self):
        res = dict(is_data=self.is_data,
                   field_name=self.field_name,
                   accepted_values=self.accepted_values)
#        if self._id:
#            res['id'] = self._id
        return res
    def filter_events(self, events):
        """ filters a collection of events according to filter values and returns
            a collection of events matching the filter 
        """
        returned_events = []
        for event in events:
            if self.is_data:
                raise Exception("code path not yet implemented")
            else:
                assert event.has_key(self.field_name), "invalid event or filter field name"
                if event[self.field_name] in self.accepted_values:
                    returned_events.append(event)
        return returned_events
    def __eq__(self, other):
        if not isinstance(other, EmailFilter):
            return False
        return self.to_dict() == other.to_dict()

    def __hash__(self):
        return hash((self.is_data, self.field_name, "_".join(self.accepted_values)))
