from .drive import Drive, DRIVE_STATES
from .physical_object import PhysicalObject
from infi.pyutils.enums import Enum, Value

ENCLOSURE_STATES = Enum(Value("ACTIVE", aliases=["OK"]),
                        Value("DEGRADED", aliases=["WARNING"]),
                        Value("FAILED", aliases=["ERROR"]),
                        "MISSING",
                        "UNKNOWN",
                      )

class Enclosure(PhysicalObject):
    _base_api_path = "components/racks/{0}/enclosures/{1}"
    states = ENCLOSURE_STATES
    def __init__(self, system, enclosure_config):
        super(Enclosure, self).__init__(system)
        self._index = enclosure_config['id']
        self._vendor = enclosure_config['vendor']
        self._drives = [Drive(system, self, drive_config)
                        for drive_config in enclosure_config['drives']]
    def get_id(self):
        return self._index
    def get_vendor(self):
        return self._vendor
    def get_drives(self):
        return self._drives[:]
    def get_drive_by_index(self, drive_index):
        filtered = [drive for drive in self._drives if drive.get_drive_index() == drive_index]
        return filtered[0] if filtered else None
    def get_running_drives(self):
        return self._get_drives_by_state(DRIVE_STATES.active)
    def _get_drives_by_state(self, state):
        return self.filter_drives(pred=lambda drive_info: drive_info['state'] == state)
    def _get_drives_list(self):
        return self._get_underlying_object()['drives']
    def filter_drives(self, pred):
        return [Drive(self._system, self, drive_config) for drive_config in self._get_drives_list() if pred(drive_config)]
    def _get_api_path(self, cmd="", *args, **kwargs):
        return super(Enclosure, self)._get_api_path(cmd, self.get_system().get_rack_id(), self.get_id(), *args, **kwargs)
    def _get_key(self):
        return (self._system, type(self), self._index)
    def __repr__(self):
        return "<Enclosure {0} of {1}>".format(self._index, self._system)

