from ..exceptions import InfinipyException
from urlobject import URLObject

class NoSuchTag(InfinipyException):
    pass

class SystemException(InfinipyException):
    pass

class AttributeNotAttached(SystemException):
    pass

class NoObjectFound(SystemException):
    pass

class AmbiguousCriteria(NoObjectFound):
    pass

class InvalidOperationException(SystemException):
    pass

class CapacityUnavailable(SystemException):
    pass

class UnkownComponentState(SystemException):
    pass

class APICommandException(SystemException):
    TEMPLATE = None
    def __init__(self, ctx):
        super(APICommandException, self).__init__()
        assert not isinstance(ctx, str)
        self.ctx = ctx
        url = URLObject(ctx.full_url)
        self.hostname = url.netloc.hostname
        self.port = url.netloc.port or (443 if url.scheme == 'https' else 80)

    def __str__(self):
        return repr(self)
    def __repr__(self):
        return self.TEMPLATE.format(e = self)

    @property
    def error_message(self):
        if not isinstance(self.ctx.output, dict):
            return None
        return self.ctx.output["error"]["message"]

    @property
    def code(self):
        if not isinstance(self.ctx.output, dict):
            return None
        return self.ctx.output['error']['code']

    message = error_message

class APICommandFailed(APICommandException):
    TEMPLATE = """API Command failed!
  {e.ctx.status_and_reason}
  Request: {e.ctx.method} {e.ctx.full_url}
  Sent headers: {e.ctx.sent_headers}
  Sent data: {e.ctx.sent_data!r}
  Output headers: {e.ctx.received_headers}
  Output: {e.ctx.raw_output!r}
"""
    @property
    def http_status_code(self):
        return self.ctx.status

class InvalidCommandOutput(APICommandException):
    TEMPLATE = """Invalid output returned from API:
  Method: {e.ctx.method}
  Request: {e.ctx.method} {e.ctx.full_url}
  Sent Data: {e.ctx.sent_data}
  Status:  {e.ctx.status_and_reason}
  Output: {e.ctx.raw_output!r}
"""

class APITransportFailure(APICommandException):
    TEMPLATE = """Failed to communicate with API:
  Method: {e.ctx.method}
  Request: {e.ctx.method} {e.ctx.full_url}
  Sent Data: {e.ctx.sent_data}
  Reason: {e.reason}
"""
    def __init__(self, ctx, reason, exception=None):
        super(APITransportFailure, self).__init__(ctx)
        self.reason = 'Unknown' if reason is None else reason
        self.exception = exception
