from infi.dtypes.wwn import WWN
from .physical_object import PhysicalObject
from infinibox_sysdefs import latest as sysdefs

FC_PORT_STATES = sysdefs.enums.nodes.fc_ports.states


class FiberPort(PhysicalObject):
    states = FC_PORT_STATES

    def __init__(self, node, fc_config):
        super(FiberPort, self).__init__(node.get_system())
        self._node = node
        self._wwpn = WWN(fc_config['wwpn'])
        self._index = fc_config['id']

    def _get_key(self):
        return (self._node, type(self), (self._index))

    def get_node(self):
        return self._node

    def get_wwpn(self):
        return self._wwpn

    def is_link_up(self):
        underlying = self._get_underlying_object()
        if underlying.get("state", "").lower() != "ok":
            return False

        link_state = underlying.get("link_state", None)
        if link_state is None:
            return True  # We cannot reliably detect link state prior to the introduction of this field

        return link_state.lower() in ("up", "link up")

    def _get_api_path(self):
        return "{0}/fc_ports/{1}".format(self._node._get_api_path(), self._index)

    def __repr__(self):
        return "<{0}:{1} <{2}> on {3!r}>".format(type(self).__name__, self._index, self._wwpn, self._node)
