from .lun_container import LUNContainer
from .system_object import CollectionSystemObject
from .defaults import has_defaults, GENERATE_NAME
from infi.dtypes.wwn import WWN

class Host(CollectionSystemObject, LUNContainer):
    SEARCHABLE_FIELD_NAMES = CollectionSystemObject.SEARCHABLE_FIELD_NAMES | set(["volume_id", "address"])
    def add_fc_port(self, port):
        return self._add_port('fc', port)
    def add_iscsi_port(self, port):
        return self._add_port('iscsi', port)
    def _add_port(self, port_type, port_address):
        return self._api.post("ports", dict(address=str(WWN(port_address)), type=port_type))
    @classmethod
    @has_defaults
    def create(cls, system, name=GENERATE_NAME):
        return cls._create_from_post(system, dict(name=name))
    def get_cluster(self):
        cluster_id = self._api.get("host_cluster_id")
        if cluster_id == 0:
            return None
        return self._system.types.Cluster(self._system, cluster_id)
    def get_fc_ports(self):
        return map(WWN, self._get_ports('fc'))
    def get_iscsi_ports(self):
        return self._get_ports('iscsi')
    def _get_ports(self, port_type):
        returned = []
        port_type = port_type.lower()
        for port in self._get_underlying_object()['ports']:
            if port_type == port['type'].lower():
                returned.append(port['address'])
        return returned
    def purge(self):
        cluster = self.get_cluster()
        if cluster is not None:
            cluster.remove_host(self)
        for lun in self.get_luns():
            self.unmap_volume(lun=lun)
        super(Host, self).purge()
    def remove_fc_port(self, port):
        return self._remove_port('fc', port)
    def remove_iscsi_port(self, port):
        return self._remove_port('iscsi', port)
    def _remove_port(self, port_type, port_address):
        self._api.delete('ports/{0}/{1}'.format(port_type, port_address))
