from munch import Munch
import httplib
from .exceptions import APICommandFailed
from .volume import Volume

class LUNContainer(object):

    def _lun_info_from_lun_dict(self, *lun_dicts):
        return dict(
            (l["lun"], Munch(
                lun = l["lun"],
                volume = Volume(self._system, l["volume_id"]),
                clustered = l["clustered"]
                ))
            for l in lun_dicts)

    def get_lun_info(self):
        """
        :returns: a dictionary mapping the lun number to an object with the following fields:
            - lun: the lun number
            - clustered: whether this lun comes from a cluster or not
            - volume: the volume for this mapping
        """
        return self._lun_info_from_lun_dict(*self._get_underlying_object()["luns"]) #gocha /luns is paged, object is not

    def get_specific_lun_info(self, lun):
        return self._lun_info_from_lun_dict(self._get_underlying_object("luns/{0}".format(lun)))[lun]

    def safe_get_specific_lun_info(self, lun):
        try:
            return self.get_specific_lun_info(lun)
        except APICommandFailed, e:
            if e.http_status_code != httplib.NOT_FOUND:
                raise

    def get_luns(self):
        return dict((lun, l.volume) for lun, l in self.get_lun_info().iteritems())

    def map_volume(self, volume, lun=None):
        data = dict(volume_id=volume.get_id())
        if lun is not None:
            data["lun"] = lun
        lun = self._api.post("luns", data)["lun"]
        return lun

    def is_volume_mapped(self, volume):
        return volume in self.get_luns().values()

    def unmap_volume(self, volume=None, lun=None, force=False):
        try:
            if volume is not None:
                self._unmap_by_volume(volume)
            if lun is not None:
                self._unmap_by_lun(lun)
        except APICommandFailed as e:
            if not force or e.ctx.status != httplib.NOT_FOUND:
                raise

    def _unmap_by_volume(self, volume):
        self._api.delete("luns/volume_id/{0}".format(volume.get_id()))

    def _unmap_by_lun(self, lun):
        self._api.delete('luns/lun/{0}'.format(lun))
