from .exceptions import AttributeNotAttached

class ExtensibleMixin(object):
    def attach_attribute(self, attr, obj):
        self._attached_attributes.add(attr)
        setattr(self, attr, obj)
    def detach_attribute(self, attr):
        if attr not in self._attached_attributes:
            raise AttributeNotAttached(attr)
        self._attached_attributes.discard(attr)
        delattr(self, attr)
    @classmethod
    def attach_method(cls, func):
        orig_func = getattr(cls, func.__name__, None)
        masked_name = '_orig_' + func.__name__
        if orig_func is not None and not hasattr(cls, masked_name) :
            setattr(cls, masked_name, orig_func)
        setattr(cls, func.__name__, func)
