from .system_object import NewAPICollectionSystemObject
import sys
from infi.pyutils.enums import Enum
from infinibox_sysdefs import latest as sysdefs


_NOTIFICATION_TARGETS_ENUMS = sysdefs.enums.notifications_targets

class NotificationTarget(NewAPICollectionSystemObject):
    protocols = _NOTIFICATION_TARGETS_ENUMS.protocols
    visibilities = _NOTIFICATION_TARGETS_ENUMS.visibilities
    snmp_auth_protocols = _NOTIFICATION_TARGETS_ENUMS.snmp_auth_protocols
    snmp_private_protocols = _NOTIFICATION_TARGETS_ENUMS.snmp_private_protocols
    snmp_auth_types = _NOTIFICATION_TARGETS_ENUMS.snmp_auth_types
    snmp_versions = _NOTIFICATION_TARGETS_ENUMS.snmp_versions
    syslog_transports = _NOTIFICATION_TARGETS_ENUMS.syslog_transports
    syslog_facilities = _NOTIFICATION_TARGETS_ENUMS.syslog_facilities

    @classmethod
    def create(cls, system, name, protocol, host, port, **kwargs):
        return cls._create_from_post(system, dict(
            name=name,
            protocol=str(protocol),
            host=host,
            port=port,
            **kwargs))

    @classmethod
    def get_collection_path(cls):
        return "notifications/targets"
    @classmethod
    def create_smtp_target(cls, system, name, host, **kwargs):
        kwargs.setdefault('port', 25)
        kwargs.setdefault('tls', False)
        kwargs.setdefault("from_address", "no-reply@infinidat.com")
        kwargs.setdefault("username", "")
        kwargs.setdefault("password", "")
        return cls.create(system, name=name, protocol=cls.protocols.smtp, host=host, **kwargs)

    @classmethod
    def create_syslog_target(cls, system, name, host, facility=None, **kwargs):
        kwargs.setdefault('transport', str(cls.syslog_transports.udp))
        kwargs.setdefault('port', 514)
        if facility is not None:
            kwargs['facility'] = str(facility)
        return cls.create(system, name=name, protocol=cls.protocols.syslog, host=host, **kwargs)
    @classmethod
    def create_snmp_target(cls, system, name, host, **kwargs):
        raise NotImplementedError("Not supported yet")
        # TODO: add defaults
        return cls.create(system, name=name, protocol=cls.protocols.snmp, host=host, **kwargs)
    @classmethod
    def get_customer_smtp_target(cls, system):
        return system.objects.NotificationTarget.get(protocol=str(cls.protocols.smtp), visibility=str(cls.visibilities.customer))
    @classmethod
    def get_infinidat_smtp_target(cls, system):
        return system.objects.NotificationTarget.get(protocol=str(cls.protocols.smtp), visibility=str(cls.visibilities.infinidat))

    @classmethod
    def get_binder_bound_method_names(cls):
        return ["create_smtp_target", "create_snmp_target", "create_syslog_target",
                "get_customer_smtp_target", "get_infinidat_smtp_target"]
    def test(self, recipients):
        self._api.post('test', data={'recipients':recipients})


    def get_visibility(self):
        return self.get_field("visibility")
