from munch import Munch
from .utils import convert_size_from_system, convert_size_for_system

class SystemCapacity(object):
    def __init__(self, system):
        super(SystemCapacity, self).__init__()
        self.system = system
    def _get_underlying_object(self):
        return self.system.api.get("system/capacity")
    def _get_capacity_dict(self):
        returned = self._get_underlying_object()
        returned = self._convert_capacities(returned)
        return returned
    def _convert_capacities(self, value):
        if not isinstance(value, dict):
            return convert_size_from_system(value)
        return Munch([(k, self._convert_capacities(v)) for k, v in value.iteritems()])
    def get_capacity_dict(self):
        returned = self._get_capacity_dict()

        returned.free_physical_capacity = returned.free_physical_space  # use same terminology
        returned.allocated_physical_capacity = returned.total_physical_capacity - returned.free_physical_capacity

        returned.free_virtual_capacity = returned.free_virtual_space  # use same terminology
        returned.allocated_virtual_capacity = returned.total_virtual_capacity - returned.free_virtual_capacity

        # reducing hierarchy of spare capacity (ignoring spare_partitions for now)
        returned.total_spare_capacity = returned.total_spare_capacity.spare_bytes if returned.has_key('total_spare_capacity') else self._convert_capacities(0)
        return returned

    def set_total_virtual_capacity(self, value):
        return convert_size_from_system(self.system.api.put('system/capacity/total_virtual_capacity', convert_size_for_system(value)))
    def get_free_physical_capacity(self):
        return self.get_capacity_dict()['free_physical_capacity']
    def get_total_physical_capacity(self):
        return self.get_capacity_dict()['total_physical_capacity']
    def get_allocated_physical_capacity(self):
        return self.get_capacity_dict()['allocated_physical_capacity']
    def get_total_spare_capacity(self):
        return self.get_capacity_dict()['total_spare_capacity']
    def get_free_virtual_capacity(self):
        return self.get_capacity_dict()['free_virtual_capacity']
    def get_total_virtual_capacity(self):
        return self.get_capacity_dict()['total_virtual_capacity']
    def get_allocated_virtual_capacity(self):
        return self.get_capacity_dict()['allocated_virtual_capacity']
    def get_spare_partitions(self):
        return self._get_underlying_object()['total_spare_capacity']['spare_partitions']
