import functools
import itertools

_BOUND_METHODS = [
    "choose",
    "create",
    "count",
    "find",
    "get",
    "recreate",
    "safe_get",
    "search",
    ]

class SystemObjectTypeBinder(object):
    def __init__(self, system, objtype):
        super(SystemObjectTypeBinder, self).__init__()
        self._system = system
        self.type = objtype
        for method_name in itertools.chain(_BOUND_METHODS, objtype.get_binder_bound_method_names()):
            method = getattr(self.type, method_name, None)
            if method is not None:
                setattr(self, method_name, functools.partial(method, self._system))
    def __call__(self, *args, **kwargs):
        return self.type(self._system, *args, **kwargs)

