from munch import Munch
from capacity import byte
from .utils import convert_size_from_system

class SystemStats(object):
    def __init__(self, system):
        super(SystemStats, self).__init__()
        self.system = system
    def get_all(self):
        return [self._convert(stat) for stat in self.system.api.get("system/stats")['system_stats']]
    def get_by_node(self, node):
        filtered = [s for s in self.get_all() if s.node_id == node.get_id()]
        return filtered[0] if filtered else None
    def get_port_stats(self, node=None):
        nodes = self.system.get_running_nodes() if node is None else [node]
        return self._sum_stats_dics([self.get_by_node(node).port_stats.connection_counters_list[0] for node in nodes])
    def _get_sub_stats(self, root_path, node=None):
        nodes = self.system.get_running_nodes() if node is None else [node]
        return self._sum_stats_dics([self.get_by_node(node)[root_path] for node in nodes])
    def get_cache_stats(self, node=None):
        return self._get_sub_stats("cache_stats", node)
    def get_destage_stats(self, node=None):
        return self._get_sub_stats("destage_stats", node)
    def get_raid_stats(self, node=None):
        return self._get_sub_stats("raid_stats", node)

    def _sum_stats_dics(self, stats_dicts):
        stast_sum = Munch()
        for k in stats_dicts[0].keys():
            stast_sum[k] = sum([d[k] for d in stats_dicts if isinstance(d[k], int)])
        return stast_sum
    def _convert(self, stats):
        SIZE_MARKER = '_bytes'
        if isinstance(stats, dict):
            converted_sizes = dict((k[:-len(SIZE_MARKER)], convert_size_from_system(v)) for k, v in stats.iteritems() if k.endswith(SIZE_MARKER))
            munch = Munch((k, self._convert(v)) for k, v in stats.iteritems())
            munch.update(**converted_sizes)
            return munch
        elif isinstance(stats, (list, tuple)):
            return type(stats)(self._convert(v) for v in stats)
        else:
            return stats

