from .physical_object import PhysicalObject
from infi.pyutils.enums import Enum, Value

UPS_STATES = Enum(Value("ACTIVE", aliases=["OK"]),
                        Value("DEGRADED", aliases=["WARNING"]),
                        Value("FAILED", aliases=["ERROR"]),
                        "MISSING",
                        "UNKNOWN",
                      )

class UPS(PhysicalObject):
    _base_api_path = "components/racks/{0}/ups/{1}"
    states = UPS_STATES
    def __init__(self, system, ups_config):
        super(UPS, self).__init__(system)
        self._index = ups_config['id']
        self._vendor = ups_config['vendor']
    def get_id(self):
        return self._index
    def get_vendor(self):
        return self._vendor
    def is_charging(self):
        return self._get_underlying_object()['charging']
    def is_on_battery(self):
        return self._get_underlying_object()['onBattery']
    def get_battery_charge(self):
        return self._get_underlying_object()['battery_charge']
    def _get_api_path(self, cmd="", *args, **kwargs):
        return super(UPS, self)._get_api_path(cmd, self.get_system().get_rack_id(), self.get_id(), *args, **kwargs)
    def _get_key(self):
        return (self._system, type(self), self._index)
    def __repr__(self):
        return "<UPS {0} of {1}>".format(self._index, self._system)

