from .system_object import CollectionSystemObject
from .system_object_type_binder import SystemObjectTypeBinder


class User(CollectionSystemObject):

    @classmethod
    def create(cls, system, username, password, role, email=None):
        return cls._create_from_post(system, {
            "username": username,
            "password": password,
            "role": role,
            "email": email
            })

    @classmethod
    def create_binder(cls, system):
        return Users(system, cls)

    def get_id(self):
        return self._id

    def get_username(self):
        return self._api.get()["username"]

    def get_email(self):
        return self._api.get()["email"]

    def get_role(self):
        return self._api.get()["role"]

    def get_pools(self):
        return [self._system.types.Pool(self._system, pool['id']) for pool in self._api.get("pools")]

    def set_email(self, email):
        self._put({"email": email})

    def set_username(self, new_name):
        self._put({'username': new_name})

    def set_password(self, password):
        self._put({"password": password})

    def set_role(self, role):
        self._put({"role": role})

    def _put(self, data):
        return self._system.api.put("users/{0}".format(self._id), data)

    def reset_password(self):
        self._api.put("reset_password", None)

    def __repr__(self):
        return "<{0} {1!r}:{2}:{3}:{4}>".format(self.__class__.__name__, self.get_system(), self.get_id(), self.get_role(), self.get_username())

    set_name = set_username
    get_name = get_username

class Users(SystemObjectTypeBinder):

    def get_users(self):
        return self.find()

    def get_by_id(self, id):
        return self._system.types.User(self._system, id)

    def get_by_email(self, email):
        return [User(self._system, u["id"]) for u in self._system.api.get('users') if u['email'] == email]

    def get_by_username(self, username):
        for user in self._system.api.get("users"):
            if user["username"] == username:
                return self._system.types.User(self._system, user["id"])
        return None
