import functools
from infi.pyutils.function_signature import FunctionSignature
from sentinels import NOTHING
from ..config import config as infinipy_config
from infi.conf.utils import get_path

class DefaultValue(object):
    def get_argument_value(self, arg_name, arguments_dict):
        raise NotImplementedError() # pragma: no cover

class FromConfiguration(DefaultValue):
    def __init__(self, path):
        super(FromConfiguration, self).__init__()
        self._path = path
    def get_argument_value(self, *_, **__):
        return get_path(infinipy_config, self._path)

def has_defaults(func):
    signature = FunctionSignature(func)
    assert not signature.is_bound_method()
    arg_names = list(signature.get_arg_names())
    args_defaults = dict((arg.name, arg.default if arg.has_default() else NOTHING) for arg in signature.get_args())
    @functools.wraps(func)
    def new_func(*args, **kwargs):
        args_dict = dict((arg_name, arg)
                         for arg_name, arg in zip(arg_names, args))
        args_dict.update(**kwargs)
        for missing in set(args_defaults) - set(args_dict):
            default = args_defaults[missing]
            if default is not NOTHING:
                if isinstance(default, DefaultValue):
                    args_dict[missing] = default.get_argument_value(missing, args_dict)
        return func(**args_dict)
    return new_func
